@@ -65,22 +65,22 @@ release_lib() {
     }
 
     echo "Push git tag $tagname"
-    echo git push ssh://git.samba.org/data/git/samba.git refs/tags/$tagname:refs/tags/$tagname || {
+    git push ssh://git.samba.org/data/git/samba.git refs/tags/$tagname:refs/tags/$tagname || {
 	exit 1
     }
 
     echo "Transferring for FTP"
-    echo rsync -Pav $tarname.asc $tgzname master.samba.org:~ftp/pub/$ftpdir/ || {
+    rsync -Pav $tarname.asc $tgzname master.samba.org:~ftp/pub/$ftpdir/ || {
 	exit 1
     }
-    exit rsync master.samba.org:~ftp/pub/$ftpdir/$tarname.*
+    rsync master.samba.org:~ftp/pub/$ftpdir/$tarname.*
 
     popd
 }
 
 for lib in $*; do
     case $lib in
-	talloc | tdb | tevent | ldb)
+	talloc | tdb | ntdb | tevent | ldb)
 	    [ -z "$GPG_USER" ] && {
 	        GPG_USER='Samba Library Distribution Key <samba-bugs@samba.org>'
 	    }
