@@ -27,7 +27,6 @@
 #include "../librpc/gen_ndr/ndr_netlogon_c.h"
 #include "../libds/common/flags.h"
 #include "ads.h"
-#include "secrets.h"
 #include "../libcli/ldap/ldap_ndr.h"
 #include "../libcli/security/security.h"
 #include "../libds/common/flag_mapping.h"
@@ -40,71 +39,213 @@
 
 extern struct winbindd_methods reconnect_methods;
 
+#define WINBIND_CCACHE_NAME "MEMORY:winbind_ccache"
+
+/**
+ * Check if cached connection can be reused. If the connection cannot
+ * be reused the ADS_STRUCT is freed and the pointer is set to NULL.
+ */
+static void ads_cached_connection_reuse(ADS_STRUCT **adsp)
+{
+
+	ADS_STRUCT *ads = *adsp;
+
+	if (ads != NULL) {
+		time_t expire;
+		time_t now = time(NULL);
+
+		expire = MIN(ads->auth.tgt_expire, ads->auth.tgs_expire);
+
+		DEBUG(7, ("Current tickets expire in %d seconds (at %d, time "
+			  "is now %d)\n", (uint32)expire - (uint32)now,
+			  (uint32) expire, (uint32) now));
+
+		if ( ads->config.realm && (expire > now)) {
+			return;
+		} else {
+			/* we own this ADS_STRUCT so make sure it goes away */
+			DEBUG(7,("Deleting expired krb5 credential cache\n"));
+			ads->is_mine = True;
+			ads_destroy( &ads );
+			ads_kdestroy(WINBIND_CCACHE_NAME);
+			*adsp = NULL;
+		}
+	}
+}
+
+/**
+ * @brief Establish a connection to a DC
+ *
+ * @param[out]   adsp             ADS_STRUCT that will be created
+ * @param[in]    target_realm     Realm of domain to connect to
+ * @param[in]    target_dom_name  'workgroup' name of domain to connect to
+ * @param[in]    ldap_server      DNS name of server to connect to
+ * @param[in]    password         Our machine acount secret
+ * @param[in]    auth_realm       Realm of local domain for creating krb token
+ * @param[in]    renewable        Renewable ticket time
+ *
+ * @return ADS_STATUS
+ */
+static ADS_STATUS ads_cached_connection_connect(ADS_STRUCT **adsp,
+						const char *target_realm,
+						const char *target_dom_name,
+						const char *ldap_server,
+						char *password,
+						char *auth_realm,
+						time_t renewable)
+{
+	ADS_STRUCT *ads;
+	ADS_STATUS status;
+	struct sockaddr_storage dc_ss;
+	fstring dc_name;
+
+	if (auth_realm == NULL) {
+		return ADS_ERROR_NT(NT_STATUS_UNSUCCESSFUL);
+	}
+
+	/* we don't want this to affect the users ccache */
+	setenv("KRB5CCNAME", WINBIND_CCACHE_NAME, 1);
+
+	ads = ads_init(target_realm, target_dom_name, ldap_server);
+	if (!ads) {
+		DEBUG(1,("ads_init for domain %s failed\n", target_dom_name));
+		return ADS_ERROR(LDAP_NO_MEMORY);
+	}
+
+	SAFE_FREE(ads->auth.password);
+	SAFE_FREE(ads->auth.realm);
+
+	ads->auth.renewable = renewable;
+	ads->auth.password = password;
+
+	ads->auth.realm = SMB_STRDUP(auth_realm);
+	if (!strupper_m(ads->auth.realm)) {
+		ads_destroy(&ads);
+		return ADS_ERROR_NT(NT_STATUS_INTERNAL_ERROR);
+	}
+
+	/* Setup the server affinity cache.  We don't reaally care
+	   about the name.  Just setup affinity and the KRB5_CONFIG
+	   file. */
+	get_dc_name(ads->server.workgroup, ads->server.realm, dc_name, &dc_ss);
+
+	status = ads_connect(ads);
+	if (!ADS_ERR_OK(status)) {
+		DEBUG(1,("ads_connect for domain %s failed: %s\n",
+			 target_dom_name, ads_errstr(status)));
+		ads_destroy(&ads);
+		return status;
+	}
+
+	/* set the flag that says we don't own the memory even
+	   though we do so that ads_destroy() won't destroy the
+	   structure we pass back by reference */
+
+	ads->is_mine = False;
+
+	*adsp = ads;
+
+	return status;
+}
+
+ADS_STATUS ads_idmap_cached_connection(ADS_STRUCT **adsp, const char *dom_name)
+{
+	char *ldap_server, *realm, *password;
+	struct winbindd_domain *wb_dom;
+	ADS_STATUS status;
+
+	ads_cached_connection_reuse(adsp);
+	if (*adsp != NULL) {
+		return ADS_SUCCESS;
+	}
+
+	/*
+	 * At this point we only have the NetBIOS domain name.
+	 * Check if we can get server nam and realm from SAF cache
+	 * and the domain list.
+	 */
+	ldap_server = saf_fetch(talloc_tos(), dom_name);
+	DEBUG(10, ("ldap_server from saf cache: '%s'\n",
+		   ldap_server ? ldap_server : ""));
+
+	wb_dom = find_domain_from_name(dom_name);
+	if (wb_dom == NULL) {
+		DEBUG(10, ("could not find domain '%s'\n", dom_name));
+		return ADS_ERROR_NT(NT_STATUS_UNSUCCESSFUL);
+	}
+
+	DEBUG(10, ("find_domain_from_name found realm '%s' for "
+			  " domain '%s'\n", wb_dom->alt_name, dom_name));
+
+	if (!get_trust_pw_clear(dom_name, &password, NULL, NULL)) {
+		TALLOC_FREE(ldap_server);
+		return ADS_ERROR_NT(NT_STATUS_CANT_ACCESS_DOMAIN_INFO);
+	}
+
+	if (IS_DC) {
+		SMB_ASSERT(wb_dom->alt_name != NULL);
+		realm = SMB_STRDUP(wb_dom->alt_name);
+	} else {
+		struct winbindd_domain *our_domain = wb_dom;
+
+		/* always give preference to the alt_name in our
+		   primary domain if possible */
+
+		if (!wb_dom->primary) {
+			our_domain = find_our_domain();
+		}
+
+		if (our_domain->alt_name != NULL) {
+			realm = SMB_STRDUP(our_domain->alt_name);
+		} else {
+			realm = SMB_STRDUP(lp_realm());
+		}
+	}
+
+	status = ads_cached_connection_connect(
+		adsp,			/* Returns ads struct. */
+		wb_dom->alt_name,	/* realm to connect to. */
+		dom_name,		/* 'workgroup' name for ads_init */
+		ldap_server,		/* DNS name to connect to. */
+		password,		/* password for auth realm. */
+		realm,			/* realm used for krb5 ticket. */
+		0);			/* renewable ticket time. */
+
+	SAFE_FREE(realm);
+	TALLOC_FREE(ldap_server);
+
+	return status;
+}
+
 /*
   return our ads connections structure for a domain. We keep the connection
   open to make things faster
 */
 static ADS_STRUCT *ads_cached_connection(struct winbindd_domain *domain)
 {
-	ADS_STRUCT *ads;
 	ADS_STATUS status;
-	fstring dc_name;
-	struct sockaddr_storage dc_ss;
+	char *password, *realm;
 
 	DEBUG(10,("ads_cached_connection\n"));
+	ads_cached_connection_reuse((ADS_STRUCT **)&domain->private_data);
 
 	if (domain->private_data) {
-
-		time_t expire;
-		time_t now = time(NULL);
-
-		/* check for a valid structure */
-		ads = (ADS_STRUCT *)domain->private_data;
-
-		expire = MIN(ads->auth.tgt_expire, ads->auth.tgs_expire);
-
-		DEBUG(7, ("Current tickets expire in %d seconds (at %d, time is now %d)\n",
-			  (uint32)expire-(uint32)now, (uint32) expire, (uint32) now));
-
-		if ( ads->config.realm && (expire > now)) {
-			return ads;
-		} else {
-			/* we own this ADS_STRUCT so make sure it goes away */
-			DEBUG(7,("Deleting expired krb5 credential cache\n"));
-			ads->is_mine = True;
-			ads_destroy( &ads );
-			ads_kdestroy("MEMORY:winbind_ccache");
-			domain->private_data = NULL;
-		}
-	}
-
-	/* we don't want this to affect the users ccache */
-	setenv("KRB5CCNAME", "MEMORY:winbind_ccache", 1);
-
-	ads = ads_init(domain->alt_name, domain->name, NULL);
-	if (!ads) {
-		DEBUG(1,("ads_init for domain %s failed\n", domain->name));
-		return NULL;
+		return (ADS_STRUCT *)domain->private_data;
 	}
 
 	/* the machine acct password might have change - fetch it every time */
 
-	SAFE_FREE(ads->auth.password);
-	SAFE_FREE(ads->auth.realm);
+	if (!get_trust_pw_clear(domain->name, &password, NULL, NULL)) {
+		return NULL;
+	}
 
 	if ( IS_DC ) {
-
-		if ( !pdb_get_trusteddom_pw( domain->name, &ads->auth.password, NULL, NULL ) ) {
-			ads_destroy( &ads );
-			return NULL;
-		}
-		ads->auth.realm = SMB_STRDUP( ads->server.realm );
-		strupper_m( ads->auth.realm );
+		SMB_ASSERT(domain->alt_name != NULL);
+		realm = SMB_STRDUP(domain->alt_name);
 	}
 	else {
 		struct winbindd_domain *our_domain = domain;
 
-		ads->auth.password = secrets_fetch_machine_password(lp_workgroup(), NULL, NULL);
 
 		/* always give preference to the alt_name in our
 		   primary domain if possible */
@@ -112,28 +253,22 @@ static ADS_STRUCT *ads_cached_connection(struct winbindd_domain *domain)
 		if ( !domain->primary )
 			our_domain = find_our_domain();
 
-		if ( our_domain->alt_name[0] != '\0' ) {
-			ads->auth.realm = SMB_STRDUP( our_domain->alt_name );
-			strupper_m( ads->auth.realm );
+		if (our_domain->alt_name != NULL) {
+			realm = SMB_STRDUP( our_domain->alt_name );
 		}
 		else
-			ads->auth.realm = SMB_STRDUP( lp_realm() );
+			realm = SMB_STRDUP( lp_realm() );
 	}
 
-	ads->auth.renewable = WINBINDD_PAM_AUTH_KRB5_RENEW_TIME;
-
-	/* Setup the server affinity cache.  We don't reaally care
-	   about the name.  Just setup affinity and the KRB5_CONFIG
-	   file. */
-
-	get_dc_name( ads->server.workgroup, ads->server.realm, dc_name, &dc_ss );
-
-	status = ads_connect(ads);
-	if (!ADS_ERR_OK(status) || !ads->config.realm) {
-		DEBUG(1,("ads_connect for domain %s failed: %s\n",
-			 domain->name, ads_errstr(status)));
-		ads_destroy(&ads);
+	status = ads_cached_connection_connect(
+					(ADS_STRUCT **)&domain->private_data,
+					domain->alt_name,
+					domain->name, NULL,
+					password, realm,
+					WINBINDD_PAM_AUTH_KRB5_RENEW_TIME);
+	SAFE_FREE(realm);
 
+	if (!ADS_ERR_OK(status)) {
 		/* if we get ECONNREFUSED then it might be a NT4
                    server, fall back to MSRPC */
 		if (status.error_type == ENUM_ADS_ERROR_SYSTEM &&
@@ -145,17 +280,9 @@ static ADS_STRUCT *ads_cached_connection(struct winbindd_domain *domain)
 		return NULL;
 	}
 
-	/* set the flag that says we don't own the memory even
-	   though we do so that ads_destroy() won't destroy the
-	   structure we pass back by reference */
-
-	ads->is_mine = False;
-
-	domain->private_data = (void *)ads;
-	return ads;
+	return (ADS_STRUCT *)domain->private_data;
 }
 
-
 /* Query display info for a realm. This is the basic user list fn */
 static NTSTATUS query_user_list(struct winbindd_domain *domain,
 			       TALLOC_CTX *mem_ctx,
@@ -191,9 +318,9 @@ static NTSTATUS query_user_list(struct winbindd_domain *domain,
 	if (!ADS_ERR_OK(rc)) {
 		DEBUG(1,("query_user_list ads_search: %s\n", ads_errstr(rc)));
 		status = ads_ntstatus(rc);
+		goto done;
 	} else if (!res) {
 		DEBUG(1,("query_user_list ads_search returned NULL res\n"));
-
 		goto done;
 	}
 
@@ -203,7 +330,7 @@ static NTSTATUS query_user_list(struct winbindd_domain *domain,
 		goto done;
 	}
 
-	(*pinfo) = TALLOC_ZERO_ARRAY(mem_ctx, struct wbint_userinfo, count);
+	(*pinfo) = talloc_zero_array(mem_ctx, struct wbint_userinfo, count);
 	if (!*pinfo) {
 		status = NT_STATUS_NO_MEMORY;
 		goto done;
@@ -223,7 +350,10 @@ static NTSTATUS query_user_list(struct winbindd_domain *domain,
 		}
 
 		info->acct_name = ads_pull_username(ads, mem_ctx, msg);
-		info->full_name = ads_pull_string(ads, mem_ctx, msg, "name");
+		info->full_name = ads_pull_string(ads, mem_ctx, msg, "displayName");
+		if (info->full_name == NULL) {
+			info->full_name = ads_pull_string(ads, mem_ctx, msg, "name");
+		}
 		info->homedir = NULL;
 		info->shell = NULL;
 		info->primary_gid = (gid_t)-1;
@@ -359,7 +489,7 @@ static NTSTATUS enum_dom_groups(struct winbindd_domain *domain,
 		goto done;
 	}
 
-	(*info) = TALLOC_ZERO_ARRAY(mem_ctx, struct wb_acct_info, count);
+	(*info) = talloc_zero_array(mem_ctx, struct wb_acct_info, count);
 	if (!*info) {
 		status = NT_STATUS_NO_MEMORY;
 		goto done;
@@ -488,7 +618,7 @@ static NTSTATUS query_user(struct winbindd_domain *domain,
 	struct netr_SamInfo3 *user = NULL;
 	gid_t gid = -1;
 	int ret;
-	char *ads_name;
+	char *full_name;
 
 	DEBUG(3,("ads: query_user\n"));
 
@@ -497,7 +627,7 @@ static NTSTATUS query_user(struct winbindd_domain *domain,
 
 	/* try netsamlogon cache first */
 
-	if ( (user = netsamlogon_cache_get( mem_ctx, sid )) != NULL ) 
+	if ( (user = netsamlogon_cache_get( mem_ctx, sid )) != NULL )
 	{
 		DEBUG(5,("query_user: Cache lookup succeeded for %s\n", 
 			 sid_string_dbg(sid)));
@@ -515,6 +645,14 @@ static NTSTATUS query_user(struct winbindd_domain *domain,
 
 		TALLOC_FREE(user);
 
+		if (info->full_name == NULL) {
+			/* this might fail so we dont check the return code */
+			wcache_query_user_fullname(domain,
+						   mem_ctx,
+						   sid,
+						   &info->full_name);
+		}
+
 		return NT_STATUS_OK;
 	}
 
@@ -592,7 +730,10 @@ static NTSTATUS query_user(struct winbindd_domain *domain,
 	 * nss_get_info_cached call. nss_get_info_cached might destroy
 	 * the ads struct, potentially invalidating the ldap message.
 	 */
-	ads_name = ads_pull_string(ads, mem_ctx, msg, "name");
+	full_name = ads_pull_string(ads, mem_ctx, msg, "displayName");
+	if (full_name == NULL) {
+		full_name = ads_pull_string(ads, mem_ctx, msg, "name");
+	}
 
 	ads_msgfree(ads, msg);
 	msg = NULL;
@@ -608,9 +749,9 @@ static NTSTATUS query_user(struct winbindd_domain *domain,
 	}
 
 	if (info->full_name == NULL) {
-		info->full_name = ads_name;
+		info->full_name = full_name;
 	} else {
-		TALLOC_FREE(ads_name);
+		TALLOC_FREE(full_name);
 	}
 
 	status = NT_STATUS_OK;
@@ -785,7 +926,7 @@ static NTSTATUS lookup_usergroups_memberof(struct winbindd_domain *domain,
 		goto done;
 	}
 
-	group_sids = TALLOC_ZERO_ARRAY(mem_ctx, struct dom_sid, num_strings + 1);
+	group_sids = talloc_zero_array(mem_ctx, struct dom_sid, num_strings + 1);
 	if (!group_sids) {
 		status = NT_STATUS_NO_MEMORY;
 		goto done;
@@ -1101,10 +1242,10 @@ static NTSTATUS lookup_groupmem(struct winbindd_domain *domain,
 	 * cache. Only the rest is passed to the lsa_lookup_sids call. */
 
 	if (num_members) {
-		(*sid_mem) = TALLOC_ZERO_ARRAY(mem_ctx, struct dom_sid, num_members);
-		(*names) = TALLOC_ZERO_ARRAY(mem_ctx, char *, num_members);
-		(*name_types) = TALLOC_ZERO_ARRAY(mem_ctx, uint32, num_members);
-		(sid_mem_nocache) = TALLOC_ZERO_ARRAY(tmp_ctx, struct dom_sid, num_members);
+		(*sid_mem) = talloc_zero_array(mem_ctx, struct dom_sid, num_members);
+		(*names) = talloc_zero_array(mem_ctx, char *, num_members);
+		(*name_types) = talloc_zero_array(mem_ctx, uint32, num_members);
+		(sid_mem_nocache) = talloc_zero_array(tmp_ctx, struct dom_sid, num_members);
 
 		if ((members == NULL) || (*sid_mem == NULL) ||
 		    (*names == NULL) || (*name_types == NULL) ||
@@ -1276,7 +1417,7 @@ static NTSTATUS sequence_number(struct winbindd_domain *domain, uint32 *seq)
 			ads = (ADS_STRUCT *)domain->private_data;
 			ads->is_mine = True;
 			ads_destroy(&ads);
-			ads_kdestroy("MEMORY:winbind_ccache");
+			ads_kdestroy(WINBIND_CCACHE_NAME);
 			domain->private_data = NULL;
 		}
 	}
@@ -1372,7 +1513,7 @@ static NTSTATUS trusted_domains(struct winbindd_domain *domain,
 		 */
 
 		if ((trust->trust_attributes
-		     == NETR_TRUST_ATTRIBUTE_QUARANTINED_DOMAIN) &&
+		     == LSA_TRUST_ATTRIBUTE_QUARANTINED_DOMAIN) &&
 		    !domain->primary )
 		{
 			DEBUG(10,("trusted_domains: Skipping external trusted "
@@ -1384,8 +1525,9 @@ static NTSTATUS trusted_domains(struct winbindd_domain *domain,
 
 		/* add to the trusted domain cache */
 
-		fstrcpy(d.name, trust->netbios_name);
-		fstrcpy(d.alt_name, trust->dns_name);
+		d.name = discard_const_p(char, trust->netbios_name);
+		d.alt_name = discard_const_p(char, trust->dns_name);
+
 		if (trust->sid) {
 			sid_copy(&d.sid, trust->sid);
 		} else {
